import math

cdef extern from "math.h":
    double sin(double)
    double cos(double)

cdef double f(double x):
    return sin(x*x) * cos(x**0.5)

def g(x):
    return math.sin(x*x) * math.cos(x**(0.5))

def integrate_fast(double a, double b, int N):
    s = 0.0
    dx = (b-a)/N
    for i in range(N):
        s += g(a+i*dx)
    return s * dx

def integrate_faster(double a, double b, int N):
    cdef double s
    cdef double dx
    cdef int i
    i = 0
    s = 0.0
    dx = (b-a)/N
    for i in range(N):
        s += f(a+i*dx)
    return s * dx

def integrate_fastest(double a, double b, int N):
    cdef double s
    cdef double dx
    cdef int i
    i = 0
    s = 0.0
    dx = (b-a)/N
    if N % 2 == 0:
        while i < N:    
            s += f(a+i*dx) + f(a+(i+1)*dx)
            i += 2
    else:
        while i < N:    
            s += f(a+i*dx)
            i += 1

    return s * dx



